
// Please see AXE133Y.h for licence info

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif
#include "AXE133Y.h"
#include "inttypes.h"

// #define bitWidth 417 //microseconds; Equates to 2400 baud
// #define bitWidth 417 //microseconds; Equates to 2400 baud
uint16_t bitWidth=417; // Modif by Yves Cordier
uint16_t interByte=500; // Modify by Yves Cordier


#define firmwareRevision "001"

// Modified librarie
// by Yves Cordier 21/10/2019
// used to debug librairie
// #define _debugAXE133YSerial

#define _nbTried (50) // sets by tried in order to get enouth precision

// Create function with multiple possibilities
AXE133Y::AXE133Y(uint8_t pin, uint16_t baud)
{
	_initialized=false;
	begin(pin,baud);
}

AXE133Y::AXE133Y(uint8_t pin)
{
	_initialized=false;
	begin(pin); // set to 2400 bauds by default
}

AXE133Y::AXE133Y()
{ 
	_initialized=false;
}

// initialized an compute delays
void AXE133Y::begin(uint8_t pin, uint16_t baud)
{   
	// modif by Yves Cordier
	// sets the speed of serial
	if (!_initialized || (baud!=_baudSpeed) || (_pin != pin) ) // if something has been modified
	{
		// sets the pin of communuication
		pinMode(pin, OUTPUT);
		_pin = pin;
		// sets the speed
		_baudSpeed=baud;
		
		// compute the delay between bitset
		byte foo = 0B10101010;
		long t2;
		long t1=micros();
		for (int i=0; i<_nbTried; i++)
		{for (byte mask = 0x01; mask; mask <<= 1) 
			{
				if (foo & mask) digitalWrite(_pin, LOW);
				else digitalWrite(_pin, HIGH); 
			}
		}
		t2=micros();
		long dt=t2-t1; // delay due to digitalWrite
		// bitWidth is the delay in microsecondes between bits to send
		bitWidth=int((1E6/_baudSpeed)-float(dt)/(8*_nbTried)); // compute the exact bitWidth 1 bit at 2400 bauds less time to send a bit
		interByte=int(0.721E6/_baudSpeed); // on attends au moins 0.721 temps d'un bit normal (300 micro secondes à 2400 bauds) au minimum avant de recommencer la danse
		_initialized=true; // job is done !!
		delay(4*interByte); // give a rest to the connected display
		
#ifdef _debugAXE133YSerial
		Serial.println(bitWidth); // for debugging libairie
		Serial.println(interByte); // idem
#endif
		// in order to clear everything spotty on the screen
		clearScreen();
	}
	// end of modif
}

// Modified function
// by Yves Cordier 21/10/2019
void AXE133Y::writeByte(uint8_t foo)
{
	if (_initialized) // no use to send something if we do not know where and how to send
	{digitalWrite(_pin, HIGH);  //start bit starts here
		delayMicroseconds(bitWidth);
		//shift a byte out to the hardware pin
		for (byte mask = 0x01; mask; mask <<= 1) {
			if (foo & mask) digitalWrite(_pin, LOW);
			else digitalWrite(_pin, HIGH); 
			delayMicroseconds(bitWidth);
		} 
		digitalWrite(_pin, LOW);  //end bit starts here
		delayMicroseconds(bitWidth);
		// delayMicroseconds(300); //this is the inter-byte delay
		// modif by Yves Cordier 21/10/2019
		delayMicroseconds(interByte);
	}
	
}

void AXE133Y::splash(String message)
{
	String splash = String("  AXE133Y V") + String(firmwareRevision);
	clearScreen();
	print(splash);
	cursorHome(2);
	print(message);
}

void AXE133Y::displayShow(bool show)
{
	writeByte(254);
	if(show) writeByte(12);
	else writeByte(8);
}

void AXE133Y::cursorShow(bool show)
{
	writeByte(254);
	if(show) writeByte(14);
	else writeByte(12);
}


void AXE133Y::cursorBlink(bool blink)
{
	writeByte(254);
	if(blink)writeByte(15);
	else writeByte(14);
}

void AXE133Y::clearScreen()
{
	writeByte(254);
	writeByte(1);  //clear
	delay(2); //wait for display to finish clearing
	writeByte(254);
	writeByte(128); //home line 1
	delay(2); //wait for command to finish
	cursorShow(0); //turn cursor off
}

void AXE133Y::cursorHome(uint8_t line)
{
	writeByte(254);
	if(line == 1) writeByte(128);
	else writeByte(192);
	delay(2); //wait for command to finish
}

void AXE133Y::cursorLeft(uint8_t moves)
{
	for (uint8_t i = 0; i<moves ; i++) 
	{
		writeByte(254);
		writeByte(16);
	}
}

void AXE133Y::cursorRight(uint8_t moves)
{
	for (uint8_t i = 0; i<moves ; i++) 
	{
		writeByte(254);
		writeByte(20);
	}
}

void AXE133Y::cursorPosition(uint8_t line, uint8_t position)
{
	writeByte(254);
	if(line ==1) position += 128;
	else position += 192;
	writeByte(position);
}

void AXE133Y::printMessage(uint8_t messageNumber)
{
	writeByte(253);
	delayMicroseconds(140); //don't take this 140us delay out
	writeByte(messageNumber);
}

void AXE133Y::print(String message)
{
	for(uint8_t i = 0; i< message.length(); i++) writeByte(message[i]);
}

void AXE133Y::outputWrite(uint8_t newState)
{
	writeByte(255);
	delayMicroseconds(250); //don't take this 250us delay out
	writeByte(newState);
}

void AXE133Y::backspace(uint8_t moves)
{
	for (int i = 0; i< moves; i++)
	{
		writeByte(254);
		writeByte(16); //cursor left
		writeByte(' ');
		writeByte(254);
		writeByte(16); //cursor left
	}
}  
// added by Yves Cordier october 2019
// the function dtostrf is available only on avr processoor
// here an equivalent for the others
// adapted from the model of Cristian Maglie <c.maglie@bug.st>

#ifndef __AVR__
char *dtostrf (double val, signed char width, unsigned char prec, char *sout) 
{
	char fmt[20]; 
	sprintf(fmt, "%%%d.%df", width, prec); // the format string
	sprintf(sout, fmt, val); // format output on sout with format string
	return sout; // could be used as function returning value or not
}
#endif

void AXE133Y::printFloat(float foo, uint8_t minimumLength, uint8_t decimalPlaces)
{
	char stringBuffer[16]; // 16 because width screen is 16
	String floatString;
	dtostrf(foo,minimumLength,decimalPlaces,stringBuffer);
	floatString = String(stringBuffer);
	print(floatString);
}
